//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: 23 December 1997
// Last Modified: 16 February 1998
// Filename:      ...sig/examples/sigmidi/improv/batonImprov/print2.cc
// Syntax:        C++; Radio Baton Improv
//  
// Description: Prints a message to the screen when a button, baton
//      or foot pedal is pressed.  Also controls the speed of printing
//	statements in mainloopalgorithm
//

#include "batonImprov.h" 

#define QUANTIZATION 100

/*----------------- beginning of improvization algorithms ---------------*/

long nextprinttime;   // time to print the statement in mainloopalgorithm
int  increment;       // time between print statements in mainloopalgorithm

/*--------------------- maintenance algorithms --------------------------*/

void description(void) {
   cout << "print --\n";
   cout << "   This program prints a statement whenever a stick, button\n";
   cout << "   or footpedal is triggered.\n";
   cout << "\n";
   cout << "   The two plus buttons control the speed of print statements\n";
   cout << "   in mainloopalgorithm:\n";
   cout << "      b14+ button will increase print time by 10 milliseconds\n";
   cout << "      b15+ button will decrease print time by 10 milliseconds\n";
   cout << "         (down to a minimum of 100 milliseconds)\n";
   cout << endl;
} 

void initialization(void) { 
   cout << "In initialization function" << endl; 
   nextprinttime = 0;
   increment = 2000;
}

void finishup(void) { 
   cout << "In cleanup function" << endl; 
}


/*-------------------- main loop algorithms -----------------------------*/

void mainloopalgorithms(void) { 
   if (t_time >= nextprinttime) {
      cout << "In main loop algorithm" << endl;
      nextprinttime = t_time + increment;       // set next print time
   }
}


/*-------------------- triggered algorithms -----------------------------*/

void stick1trig(void) { 
   cout << "\nStick 1 triggered" << endl; 
}

void stick2trig(void) { 
   cout << "\nStick 2 triggered" << endl; 
}

void b14plustrig(void) { 
   cout << "\nb14+ button pushed" << endl; 
   increment += QUANTIZATION;
}

void b15plustrig(void) { 
   cout << "\nb15+ button pushed" << endl; 
   increment -= QUANTIZATION;
   if (increment < 100)  increment = 100;
}

void b14minusuptrig(void) { 
   cout << "\nb14- pedal depressed" << endl; 
}

void b14minusdowntrig(void) { 
   cout << "\nb14- pedal released" << endl; 
}

void b15minusuptrig(void) { 
   cout << "\nb15- pedal depressed" << endl; 
}

void b15minusdowntrig(void) { 
   cout << "\nb15- pedal released" << endl; 
}

void keyboardchar(int key) { 
   cout << "A character was received by keyboardchar: " << key << endl;
}


/*------------------ end improvization algorithms -----------------------*/



