//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Tue Mar 10 21:49:05 GMT-0800 1998
// Last Modified: Tue Mar 10 21:49:08 GMT-0800 1998
// Filename:      ...sig/examples/sigmidi/improv/batonImprov/taptempo.cc
// Syntax:        C++; Radio Baton Improv
//  
// Description: Fill in the following functions to create an improv program.
//

#include "batonImprov.h" 


/*----------------- beginning of improvization algorithms ---------------*/

int lastBeatTime = 0;      // time in milliseconds of the last beat
int currentBeatTime = 0;   // time in milliseconds of the current beat
int quarterDur = 0;        // current duration of quarternotes in milliseconds
int quarterTicks = 0;      // number of ticks per quarternote in the MIDI file
int currTrackTick = 0;     // conv current time to MIDI track clock ticks count
MidiFile midiTracks;       // MIDI file storage variable
int trackCount = 0;        // number of tracks in the MIDI file
MidiMessage* trackMessage = NULL; // storage of next track message to play
int* trackActionTick = NULL;      // storage for next time to play a track event
int running = 0;           // true if currently playing a MIDI file
int startTime = 0;         // the start time of the MIDI file in milliseconds
CTimer globalTime;         // current time in milliseconds
int beatCount = -1;        // the number of the beat currently on in MIDI file.


/*--------------------- maintenance algorithms --------------------------*/

void description(void) {
   cout << "Write the program description here" << endl;
   cout << "B14 button starts" << endl;
   cout << "Trigger with baton1 to control tempo and play MIDI file" << endl;
   cout << "B15 button resets" << endl;
} 


void initialization(void) { 
   cout << "Enter the name of the MIDI file to play: ";
   char filename[1000];
   cin >> filename;
   midiTracks.input(filename);
   trackCount = midiTracks.getNumTracks();
   quarterTicks = midiTracks.getTicksPerQuarterNote();
   if (trackCount <= 0) {
      cerr << "Invalid number of tracks: " << trackCount << endl;
      exit(1);
   }   

   trackMessage = new MidiMessage[trackCount];
   trackActionTick = new int[trackCount];

   for (int i=0; i<trackCount; i++) {
      trackMessage[i] = midiTracks.getNextMessage(i);
      trackActionTick[i] = trackMessage[i].time;
   }
}


void finishup(void) { 
   delete [] trackMessage;
   delete [] trackActionTick;
}


/*-------------------- main loop algorithms -----------------------------*/

void checkTracks(void);
void updateTrack(int aTrack);
int  calculateCurrentMIDIFileTick();


void mainloopalgorithms(void) { 
   if (running == 1) {
      checkTracks();
   }
}



//////////////////////////////
//
// checkTracks -- check to see if it is time to play a MidiEvent 
//    in a track
//

void checkTracks(void) {
   currTrackTick = calculateCurrentMIDIFileTick();
   for (int i=0; i<trackCount; i++) {
      if (trackActionTick[i] <= currTrackTick)   updateTrack(i);
   }
}



///////////////////////////////
//
// calculateCurrentMIDIFileTick -- convert the current time
//    into MIDI time clock ticks in MIDI file.
//

int calculateCurrentMIDIFileTick(void) {
   double beatFraction;
   beatFraction = ((double)(globalTime.getTime() - currentBeatTime))/quarterDur;
   return (int)((beatCount + beatFraction) * quarterTicks);    
}



///////////////////////////////
//
// updateTrack -- play MIDI command in track and set time for next
//    message to be played
//

void updateTrack(int aTrack) {
   while (currTrackTick <= trackActionTick[aTrack]) {
      synth.rawsend(trackMessage[aTrack].command(),
         trackMessage[aTrack].p1(), trackMessage[aTrack].p2());
      trackMessage[aTrack] = midiTracks.getNextMessage(aTrack);
      trackActionTick[aTrack] += trackMessage[aTrack].time;
   }
}



/*-------------------- triggered algorithms -----------------------------*/

void stick1trig(void) { 
   if (startTime == 0)  startTime = globalTime.getTime();
   lastBeatTime = currentBeatTime;
   currentBeatTime = baton.t1t[0];
   quarterDur = currentBeatTime - lastBeatTime;
   beatCount++;
   if (quarterDur == 0)  quarterDur = 1000;
}


void stick2trig(void) { }

void b14plustrig(void) { 
   running = 1;      // start playing MIDI file when b14+ button pressed
}

void b15plustrig(void) {  // reset to initial state
   running = 0;
   startTime = 0;
   beatCount = -1;
   quarterDur = 0;
   midiTracks.resetAllTracks();
   currentBeatTime = 0;
   lastBeatTime = 0;

   for (int i=0; i<trackCount; i++) {
      trackMessage[i] = midiTracks.getNextMessage(i);
      trackActionTick[i] = trackMessage[i].time;
   }

}


void b14minusuptrig(void) { }

void b14minusdowntrig(void) { }

void b15minusuptrig(void) { }

void b15minusdowntrig(void) { }

void keyboardchar(int key) { }


/*------------------ end improvization algorithms -----------------------*/
