//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: 5 January 1998
// Last Modified: Thu Feb 12 17:09:15 GMT-0800 1998
// Filename:      ...sig/examples/sigmidi/improv/synthImprov/shadow.cpp
// Syntax:        C++; Synth Improv
//  
// Description: echos a note played on the synthesizer keyboard
//

#include "synthImprov.h" 


/*----------------- beginning of improvization algorithms ---------------*/

int shadowDistance = 12;     // number of half-steps to place shadow at
int shadowSide     = 1;      // 1 = higher, -1 = lower
MidiMessage noteMessage;     // for reading keyno and velocity (and time)
int shadowNote = 0;          // note to play as a shadow
int octave = 4;              // octave number of computer keyboard notes


/*--------------------- maintenance algorithms --------------------------*/


void description(void) {
   cout << endl;
   printboxtop();
   psl(" Shadow -- by Craig Stuart Sapp <craig@ccrma.stanford.edu>,  5 Jan 1998");
   printintermediateline();
   psl("   This program will echo the input notes at a certain distance from");
   psl("   the original notes.  The range of the shadow is from 1 to 24");
   psl("   half-steps on either side of the original note.");
   printintermediateline();
   psl("  User commands:");
   psl("     \"-\" = decrease shadow length   \"=\" = increase shadow length ");
   psl("     \"\\\" = switch shadow side    ");
   psl("");
   psl("  Computer synthesizer keyboard:");
   psl("      \"0\"-\"9\" = octave number of computer keyboard notes");
   psl("      Notes:           s   d      g    h   j   ");
   psl("                     z   x   c   v   b   n   m  ");
   printboxbottom();
   cout << endl;
} 


void initialization(void) { }


void finishup(void) { }



/*-------------------- main loop algorithms -----------------------------*/

void mainloopalgorithms(void) { 
   if (synth.getNoteCount() > 0) {
      noteMessage = synth.extractNote();
      if (noteMessage.p2() != 0) {              // ignore note off commands
         synth.play(CH_1, shadowNote, 0);          // turn off last note
         shadowNote = noteMessage.p1() + shadowSide * shadowDistance;
         if (shadowNote > 0 && shadowNote < 128) {
            synth.play(CH_1, shadowNote, noteMessage.p2());
         } else {
            shadowNote = 0;
         }
      }
   }
}



/*-------------------- triggered algorithms -----------------------------*/

void keyboardchar(int key) { 
   if (isdigit((char)key)) {
      octave = key - '0';
      return;
   }
   switch (key) {
      case '-': 
         shadowDistance--; 
         if (shadowDistance < 1) shadowDistance = 1;
         cout << "\r Shadow = " << shadowSide * shadowDistance << "\t\t\t" << flush;
         break;
      case '=':
         shadowDistance++; 
         if (shadowDistance > 24) shadowDistance = 24;
         cout << "\r Shadow = " << shadowSide * shadowDistance << "\t\t\t" << flush;
         break;
      case '\\':
         shadowSide *= -1;
         cout << "\r Shadow = " << shadowSide * shadowDistance << "\t\t\t" << flush;
         break;
      default:
         octavekeyboard(key, octave);
   }
}



/*------------------ end improvization algorithms -----------------------*/



